package jaccard;

import java.awt.Container;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import icy.common.exception.UnsupportedFormatException;
import icy.file.Loader;
import icy.file.SequenceFileImporter;
import icy.gui.frame.IcyFrame;
import icy.gui.frame.IcyFrameAdapter;
import icy.gui.frame.IcyFrameEvent;
import icy.image.IcyBufferedImage;
import icy.image.IcyBufferedImageUtil;
import icy.main.Icy;
import icy.plugin.abstract_.PluginActionable;
import icy.sequence.Sequence;
import icy.type.DataType;
import ij.ImagePlus;
import ij.gui.PolygonRoi;
import ij.gui.Roi;
import ij.process.ByteProcessor;
import plugins.big.bigsnakeutils.icy.snake2D.Snake2DScale;


/**
 * This class handle the computation of the Jaccard index
 * @author Yann Perret (yann.perret@epfl.ch)
 * 2016-2017
 */
public class JaccardSnakes extends PluginActionable{
	
	//Groundtruth (mask)
	private double[][] mask = null;

	//Skin of the snake
	private Snake2DScale[] skin1 = null;
	
	private double jaccardIndex_=0;
	
	
	//GUI
	private final JLabel loadGSButton_ = new JLabel("Ground Truth Mask");
	private final JButton browseButton1_ = new JButton("Browse");
	private final JButton runButton_ = new JButton("Run");
	private final JLabel jaccardIndexLabel_=new JLabel("Jaccard index");
	private final JLabel jaccardIndexValueLabel_=new JLabel("                 ");
	private final JLabel dataTitleLabel_=new JLabel("Data");
	private final JLabel resultLabel_=new JLabel("Result");
	private final IcyFrame plugInMainFrame_ = new IcyFrame();


	public JaccardSnakes(Snake2DScale[] skin) {
		this.skin1 = skin;
	}

	public void run() {
		runInterface();
	}
	
	private void runInterface() {
		Container mainPane = plugInMainFrame_.getContentPane();
		
		Settings settings = Settings.getInstance();
		plugInMainFrame_.setTitle(settings.getAppName() + " " + settings.getAppVersion());

		mainPane.setLayout(new BoxLayout(mainPane, BoxLayout.PAGE_AXIS));
		plugInMainFrame_.pack();

		plugInMainFrame_.addFrameListener(new IcyFrameAdapter() {
			@Override
			public void icyFrameInternalized(IcyFrameEvent e) {
				plugInMainFrame_.pack();
			}

			@Override
			public void icyFrameExternalized(IcyFrameEvent e) {
				plugInMainFrame_.pack();
			}
		});
		
		
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[] {90, 100, 0 };
		gridBagLayout.rowHeights = new int[] { 27,2,27,2, 27,2, 27,27, 2,0 };
		gridBagLayout.columnWeights = new double[] { 0.0, 1.0, Double.MIN_VALUE };
		gridBagLayout.rowWeights = new double[] { 0.0, 0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0, Double.MIN_VALUE };
		mainPane.setLayout(gridBagLayout);
		
		JPanel dataTitleLabelPanel = new JPanel();
		GridBagConstraints gbc_dataParametersTitleLabelPanel = new GridBagConstraints();
		gbc_dataParametersTitleLabelPanel.anchor = GridBagConstraints.WEST;
		gbc_dataParametersTitleLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_dataParametersTitleLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_dataParametersTitleLabelPanel.gridx = 0;
		gbc_dataParametersTitleLabelPanel.gridy = 0;
		mainPane.add(dataTitleLabelPanel, gbc_dataParametersTitleLabelPanel);
		dataTitleLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		dataTitleLabelPanel.add(dataTitleLabel_);
		dataTitleLabel_.setFont(new Font("", Font.BOLD, 13));

		JPanel interactionPanel = new JPanel();
		GridBagConstraints gbc_interactionPanel = new GridBagConstraints();
		gbc_interactionPanel.anchor = GridBagConstraints.EAST;
		gbc_interactionPanel.insets = new Insets(0, 0, 5, 5);
		gbc_interactionPanel.fill = GridBagConstraints.VERTICAL;
		gbc_interactionPanel.gridx = 0;
		gbc_interactionPanel.gridy = 2;
		mainPane.add(interactionPanel, gbc_interactionPanel);
		interactionPanel.setLayout(new GridLayout(0, 1, 0, 0));		
		interactionPanel.add(loadGSButton_);
		
		JPanel browsePanel = new JPanel();
		GridBagConstraints gbc_browsePanel = new GridBagConstraints();
		gbc_browsePanel.anchor = GridBagConstraints.EAST;
		gbc_browsePanel.insets = new Insets(0, 0, 5, 5);
		gbc_browsePanel.fill = GridBagConstraints.VERTICAL;
		gbc_browsePanel.gridx = 1;
		gbc_browsePanel.gridy = 2;
		mainPane.add(browsePanel, gbc_browsePanel);
		browsePanel.setLayout(new GridLayout(0, 1, 0, 0));		
		browsePanel.add(browseButton1_);
		
		JPanel runPanel = new JPanel();
		GridBagConstraints gbc_runPanel = new GridBagConstraints();
		gbc_runPanel.anchor = GridBagConstraints.EAST;
		gbc_runPanel.insets = new Insets(0, 0, 5, 5);
		gbc_runPanel.fill = GridBagConstraints.VERTICAL;
		gbc_runPanel.gridx = 1;
		gbc_runPanel.gridy = 6;
		mainPane.add(runPanel, gbc_runPanel);
		runPanel.setLayout(new GridLayout(0, 1, 0, 0));		
		runPanel.add(runButton_);
		
		JPanel resultLabelPanel = new JPanel();
		GridBagConstraints gbc_resultsLabelPanel = new GridBagConstraints();
		gbc_resultsLabelPanel.anchor = GridBagConstraints.WEST;
		gbc_resultsLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_resultsLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_resultsLabelPanel.gridx = 0;
		gbc_resultsLabelPanel.gridy = 7;
		mainPane.add(resultLabelPanel, gbc_resultsLabelPanel);
		resultLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		resultLabelPanel.add(resultLabel_);
		resultLabel_.setFont(new Font("", Font.BOLD, 13));
		
		JPanel jaccardPanel = new JPanel();
		GridBagConstraints gbc_jaccardPanel = new GridBagConstraints();
		gbc_jaccardPanel.anchor = GridBagConstraints.EAST;
		gbc_jaccardPanel.insets = new Insets(0, 0, 5, 5);
		gbc_jaccardPanel.fill = GridBagConstraints.VERTICAL;
		gbc_jaccardPanel.gridx = 0;
		gbc_jaccardPanel.gridy = 9;
		mainPane.add(jaccardPanel, gbc_jaccardPanel);
		jaccardPanel.setLayout(new GridLayout(0, 1, 0, 0));		
		jaccardPanel.add(jaccardIndexLabel_);
		
		JPanel jaccardValuePanel = new JPanel();
		GridBagConstraints gbc_jaccardValuePanel = new GridBagConstraints();
		gbc_jaccardValuePanel.anchor = GridBagConstraints.EAST;
		gbc_jaccardValuePanel.insets = new Insets(0, 0, 5, 5);
		gbc_jaccardValuePanel.fill = GridBagConstraints.VERTICAL;
		gbc_jaccardValuePanel.gridx = 1;
		gbc_jaccardValuePanel.gridy = 9;
		mainPane.add(jaccardValuePanel, gbc_jaccardValuePanel);
		jaccardValuePanel.setLayout(new GridLayout(0, 1, 0, 0));		
		jaccardValuePanel.add(jaccardIndexValueLabel_);
		jaccardIndexValueLabel_.setSize(10,10);
		jaccardIndexValueLabel_.setBorder(BorderFactory.createEtchedBorder());
		
		//Action
		browseButton1_.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				loadMask();
			}
		});
		
		runButton_.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				jaccardIndex_= computeJaccard(skin1[0], mask);
				jaccardIndexValueLabel_.setText(""+jaccardIndex_);
			}
		});
		
		//mainPane.add(interactionPanel);

		plugInMainFrame_.pack();

		plugInMainFrame_.addFrameListener(new IcyFrameAdapter() {
			@Override
			public void icyFrameInternalized(IcyFrameEvent e) {
				plugInMainFrame_.pack();
			}

			@Override
			public void icyFrameExternalized(IcyFrameEvent e) {
				plugInMainFrame_.pack();
			}
		});

		addIcyFrame(plugInMainFrame_);
		plugInMainFrame_.center();
		plugInMainFrame_.setVisible(true);
	}


	/**
	 * Compute the jaccard index between a snake skin and a binary mask
	 * @param skin1 skin of the snake
	 * @param mask binary mask as 2D double image
	 * @return jaccard index
	 */
	public double computeJaccard(Snake2DScale skin1, double[][] mask) {
		double jaccard = 0.0;
		double and = 0.0;
		double or = 0.0;

		PolygonRoi roi1 = new PolygonRoi((skin1.xpoints).clone(),
				(skin1.ypoints).clone(), skin1.npoints, Roi.TRACED_ROI);
		if (roi1.getMask() == null /*|| roi2.getMask() == null*/) {
			return 0.0;
		}
		

		ByteProcessor bp1 = new ByteProcessor(mask.length, mask[0].length);
		bp1.insert(roi1.getMask(), roi1.getBounds().x, roi1.getBounds().y);
		
		//To display the binary mask of the snake
//		Sequence sequence = new Sequence();
//		
//		IcyBufferedImage tempImage = new IcyBufferedImage(mask.length, mask[0].length, 1, DataType.DOUBLE);
//		double[] temp = tempImage.getDataXYAsDouble(0);
//			
//		for (int y = 0; y < mask.length; y++) {
//			for (int x = 0; x < mask[0].length; x++) {
//				temp[x + y * mask.length] = bp1.getPixel(x, y);
//			}
//		}
//			
//		tempImage.dataChanged();
//		sequence.addImage(tempImage);
//		
//		Icy.getMainInterface().addSequence(sequence);
		//

		double a = 0.0, b = 0.0;
		for (int x = 0; x < mask.length; x++) {
			for (int y = 0; y < mask[0].length; y++) {
				a = bp1.getPixel(x, y);
				b = mask[x][y];			
				
				if (a != 0 || b != 0) {
					or += 1.0;
				}
				if (a != 0 && b != 0) {
					and += 1.0;
				}
			}
		}
		
		jaccard = and / or;
		return jaccard;
	}
	
	public void loadMask(){
		String userDirLocation = "/Users/anais/switchdrive/Projets/Texture Snakes/My_projet/Experiments/Real_Images/Masks";
		File userDir = new File(userDirLocation);
		JFileChooser fileChooser = new JFileChooser(userDir);
		
		fileChooser.showOpenDialog(new JFrame());
		String filename = fileChooser.getSelectedFile().getAbsolutePath();
		
		if (filename.endsWith("Store")==false){
			
		    SequenceFileImporter importer = Loader.getSequenceFileImporter(filename, false);   
		    
		    try {
				if (importer.open(filename,0)) {
				    IcyBufferedImage img = IcyBufferedImageUtil.convertToType(importer.getImage(0, 0),DataType.DOUBLE, false);
				    BufferedImage im=img;
				    ImagePlus imPlus=new ImagePlus("image",im);
				    this.mask=ImagePlus2Array2D(imPlus);
				}
			} catch (UnsupportedFormatException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		    
		}
	}
	
	public static double[][] ImagePlus2Array2D(ImagePlus imp) {

		int nx = imp.getWidth();
		int ny = imp.getHeight();

		double[][] output = new double[nx][ny];

		for(int y = 0; y < ny; y++) {
			for(int x = 0; x < nx; x++) {
				output[x][y] = imp.getProcessor().getPixelValue(x, y);

			}
		}

		return output;
	}
}
