/*******************************************************************************
 * Copyright (c) 2012-2013 Biomedical Image Group (BIG), EPFL, Switzerland.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Ricard Delgado-Gonzalo (ricard.delgado@gmail.com)
 *     Nicolas Chenouard (nicolas.chenouard@gmail.com)
 *     Philippe Th&#233;venaz (philippe.thevenaz@epfl.ch)
 *     Emrah Bostan (emrah.bostan@gmail.com)
 *     Ulugbek S. Kamilov (kamilov@gmail.com)
 *     Ramtin Madani (ramtin_madani@yahoo.com)
 *     Masih Nilchian (masih_n85@yahoo.com)
 *     C&#233;dric Vonesch (cedric.vonesch@epfl.ch)
 *     Virginie Uhlmann (virginie.uhlmann@epfl.ch)
 *     Cl&#233;ment Marti (clement.marti@epfl.ch)
 *     Julien Jacquemot (julien.jacquemot@epfl.ch)
 ******************************************************************************/
package plugins.big.texturesnake.gui;

import icy.canvas.Canvas2D;
import icy.canvas.IcyCanvas;
import icy.image.IcyBufferedImage;
import icy.image.IcyBufferedImageUtil;
import icy.main.Icy;
import icy.sequence.Sequence;
import icy.type.DataType;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.IOException;
import java.util.HashMap;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import plugins.big.bigsnakeutils.icy.gui.CollapsiblePanel;
import plugins.big.bigsnakeutils.icy.gui.DetailPanelMode;
import plugins.big.texturesnake.core.ImageLUTContainer;
import plugins.kernel.roi.roi2d.ROI2DRectangle;

/**
 * Panel in which the user specifies image-related parameters.
 * 
 * @version February 8, 2015
 * 
 * @author Ricard Delgado-Gonzalo (ricard.delgado@gmail.com)
 */
public class ImageSettingsPane extends CollapsiblePanel implements
		ActionListener, ChangeListener, MouseListener, MouseMotionListener {

	/**
	 * Determines if a de-serialized file is compatible with this class.
	 * 
	 * Maintainers must change this value if and only if the new version of this
	 * class is not compatible with old versions. See Sun docs for <a
	 * href=http://java.sun.com/products/jdk/1.1/docs/guide
	 * /serialization/spec/version.doc.html> details. </a>
	 * 
	 * Not necessary to include in first version of the class, but included here
	 * as a reminder of its importance.
	 */
	private static final long serialVersionUID = 1437522417562022731L;
	private int textureWidth_=95;
	private int textureHeight_=95;
	private int iconSizeWidth_=95;
	private int iconSizeHeight_=95;
	private IcyBufferedImage texture1_=new IcyBufferedImage(textureWidth_, textureHeight_, 1, DataType.DOUBLE) ;
	private IcyBufferedImage texture2_=new IcyBufferedImage(textureWidth_, textureHeight_, 1, DataType.DOUBLE) ;
	ROI2DRectangle rect_=null;
	ROI2DRectangle ROITexture1_=null;
	ROI2DRectangle ROITexture2_=null;
	boolean oneTextureAsBeenValidated_=false;


	// ----------------------------------------------------------------------------
	// LABELS

	/** Label for the selected image combo box. */
	private final JLabel selectedImageLabel_ = new JLabel("Image");
	/** Label displaying the name of the selected image. */
	private final JLabel selectedImageName_ = new JLabel("");
	
	
	private final JLabel numScalesLabel_ = new JLabel("Scales");
	private final JLabel numHarmonicsLabel_ = new JLabel("Harmonics");
	private final JLabel texture1Label_ = new JLabel("Texture 1");
	private final JLabel texture2Label_ = new JLabel("Texture 2");
	private final JLabel imageData_=new JLabel("Image data");
	private final JLabel textureParametersTitleLabel_=new JLabel("Texture parameters");
	
	private final JSpinner numScalesSpinner_ = new JSpinner();
	private final JSpinner numHarmonicsSpinner_ = new JSpinner();
	private final JButton  texture1SelectButton_=new JButton("Select");
	private final JButton  texture1ValidateButton_=new JButton("Ok");
	private final JButton  texture2SelectButton_=new JButton("Select");
	private final JButton  texture2ValidateButton_=new JButton("Ok");
	
	private JLabel texture1ImageLabel_ = new JLabel(new ImageIcon(texture1_.getScaledInstance(textureWidth_, textureHeight_, 0)));
	private JLabel texture2ImageLabel_ = new JLabel(new ImageIcon(texture2_.getScaledInstance(textureWidth_, textureHeight_, 0)));
	
	private JPanel texture1ImagePanel_ = new JPanel();
	private JPanel texture2ImagePanel_ = new JPanel();

	 
	// ----------------------------------------------------------------------------
	// INTERACTIVE INPUT EVELENTS

	/**
	 * Points to the HashMap of the plug-in that pairs the images and the LUTs.
	 */
	private HashMap<Sequence, ImageLUTContainer> imageLUTs_ = null;

	/** Focused image. */
	private Sequence activeSequence_ = null;

	// ============================================================================
	// PUBLIC METHODS

	/** Constructor. 
	 * @throws IOException */
	public ImageSettingsPane(String title,
			HashMap<Sequence, ImageLUTContainer> imageLUTs) throws IOException {
		super(title);
		imageLUTs_ = imageLUTs;

		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[] { 150, 150, 120 };
		gridBagLayout.rowHeights = new int[] { 27,27,27,27,27,27,27,27,27,27,27};
		setLayout(gridBagLayout);
		
		//Title:image data
		JPanel imageDataTitleLabelPanel = new JPanel();
		GridBagConstraints gbc_imageDataTitleLabelPanel = new GridBagConstraints();
		gbc_imageDataTitleLabelPanel.anchor = GridBagConstraints.WEST;
		gbc_imageDataTitleLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_imageDataTitleLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_imageDataTitleLabelPanel.gridx = 0;
		gbc_imageDataTitleLabelPanel.gridy = 0;
		add(imageDataTitleLabelPanel, gbc_imageDataTitleLabelPanel,
				DetailPanelMode.BASICS);
		imageDataTitleLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		imageDataTitleLabelPanel.add(imageData_);
		imageData_.setFont(new Font("", Font.BOLD, 13));
		
		
		//Image
		JPanel selectedImageLabelPanel = new JPanel();
		GridBagConstraints gbc_panel_3 = new GridBagConstraints();
		gbc_panel_3.anchor = GridBagConstraints.EAST;
		gbc_panel_3.insets = new Insets(0, 0, 5, 5);
		gbc_panel_3.fill = GridBagConstraints.VERTICAL;
		gbc_panel_3.gridx = 0;
		gbc_panel_3.gridy = 1;
		add(selectedImageLabelPanel, gbc_panel_3, DetailPanelMode.BASICS);
		selectedImageLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		selectedImageLabelPanel.add(selectedImageLabel_);

		JPanel selectedImageComboBoxPanel = new JPanel();
		GridBagConstraints gbc_panel = new GridBagConstraints();
		gbc_panel.insets = new Insets(0, 0, 5, 0);
		gbc_panel.fill = GridBagConstraints.BOTH;
		gbc_panel.gridx = 1;
		gbc_panel.gridy = 1;
		add(selectedImageComboBoxPanel, gbc_panel, DetailPanelMode.BASICS);
		selectedImageComboBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));

		selectedImageComboBoxPanel.add(selectedImageName_);
	
		//Title: texture parameters
		JPanel textureParametersTitleLabelPanel = new JPanel();
		GridBagConstraints gbc_textureParametersTitleLabelPanel = new GridBagConstraints();
		gbc_textureParametersTitleLabelPanel.anchor = GridBagConstraints.WEST;
		gbc_textureParametersTitleLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_textureParametersTitleLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_textureParametersTitleLabelPanel.gridx = 0;
		gbc_textureParametersTitleLabelPanel.gridy = 3;
		add(textureParametersTitleLabelPanel, gbc_textureParametersTitleLabelPanel,
				DetailPanelMode.BASICS);
		textureParametersTitleLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		textureParametersTitleLabelPanel.add(textureParametersTitleLabel_);
		textureParametersTitleLabel_.setFont(new Font("", Font.BOLD, 13));
		
		
		//Scales
		JPanel numScalesLabelPanel = new JPanel();
		GridBagConstraints gbc_numScalesLabelPanel = new GridBagConstraints();
		gbc_numScalesLabelPanel.anchor = GridBagConstraints.EAST;
		gbc_numScalesLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_numScalesLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_numScalesLabelPanel.gridx = 0;
		gbc_numScalesLabelPanel.gridy = 4;
		add(numScalesLabelPanel, gbc_numScalesLabelPanel,
				DetailPanelMode.BASICS);
		numScalesLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		numScalesLabelPanel.add(numScalesLabel_);
		
		JPanel numScalesSpinnerPanel = new JPanel();
		GridBagConstraints gbc_numScalesSpinnerPanel = new GridBagConstraints();
		gbc_numScalesSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbc_numScalesSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbc_numScalesSpinnerPanel.gridx = 1;
		gbc_numScalesSpinnerPanel.gridy = 4;
		add(numScalesSpinnerPanel, gbc_numScalesSpinnerPanel,
				DetailPanelMode.BASICS);
		numScalesSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));

		numScalesSpinnerPanel.add(numScalesSpinner_);
		numScalesSpinner_.setModel(new SpinnerNumberModel(new Integer(
				3), new Integer(0), new Integer(10),
				new Integer(1)));
		
		//Harmonics
		JPanel numHarmonicsLabelPanel = new JPanel();
		GridBagConstraints gbc_numHarmonicsLabelPanel = new GridBagConstraints();
		gbc_numHarmonicsLabelPanel.anchor = GridBagConstraints.EAST;
		gbc_numHarmonicsLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_numHarmonicsLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_numHarmonicsLabelPanel.gridx = 0;
		gbc_numHarmonicsLabelPanel.gridy = 5;
		add(numHarmonicsLabelPanel, gbc_numHarmonicsLabelPanel,
				DetailPanelMode.BASICS);
		numHarmonicsLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		numHarmonicsLabelPanel.add(numHarmonicsLabel_);
		
		JPanel numHarmonicsSpinnerPanel = new JPanel();
		GridBagConstraints gbc_numHarmonicsSpinnerPanel = new GridBagConstraints();
		gbc_numHarmonicsSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbc_numHarmonicsSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbc_numHarmonicsSpinnerPanel.gridx = 1;
		gbc_numHarmonicsSpinnerPanel.gridy = 5;
		add(numHarmonicsSpinnerPanel, gbc_numHarmonicsSpinnerPanel,
				DetailPanelMode.BASICS);
		numHarmonicsSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));

		numHarmonicsSpinnerPanel.add(numHarmonicsSpinner_);
		numHarmonicsSpinner_.setModel(new SpinnerNumberModel(new Integer(
				5), new Integer(1), new Integer(10),
				new Integer(1)));	
		
		
		//Texture 1
		JPanel texture1LabelPanel = new JPanel();
		GridBagConstraints gbc_texture1LabelPanel = new GridBagConstraints();
		gbc_texture1LabelPanel.anchor = GridBagConstraints.EAST;
		gbc_texture1LabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_texture1LabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_texture1LabelPanel.gridx = 0;
		gbc_texture1LabelPanel.gridy = 7;
		add(texture1LabelPanel, gbc_texture1LabelPanel,
				DetailPanelMode.BASICS);
		texture1LabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		texture1LabelPanel.add(texture1Label_);
		
		JPanel texture1SelectButtonPanel = new JPanel();
		GridBagConstraints gbc_texture1SelectButtonPanel = new GridBagConstraints();
		gbc_texture1SelectButtonPanel.insets = new Insets(0, 0, 5, 0);
		gbc_texture1SelectButtonPanel.fill = GridBagConstraints.BOTH;
		gbc_texture1SelectButtonPanel.gridx = 1;
		gbc_texture1SelectButtonPanel.gridy = 7;
		add(texture1SelectButtonPanel, gbc_texture1SelectButtonPanel,
				DetailPanelMode.BASICS);
		texture1SelectButtonPanel.setLayout(new GridLayout(2, 1, 0, 0));
			GridBagConstraints grid1 = new GridBagConstraints();
			grid1.insets = new Insets(0, 0, 5, 0);
			grid1.fill = GridBagConstraints.BOTH;
			grid1.gridx = 0;
			grid1.gridy = 0;
		texture1SelectButtonPanel.add(texture1SelectButton_, grid1);
			GridBagConstraints grid2 = new GridBagConstraints();
			grid2.insets = new Insets(0, 0, 5, 0);
			grid2.fill = GridBagConstraints.BOTH;
			grid2.gridx = 1;
			grid2.gridy = 0;
		texture1SelectButtonPanel.add(texture1ValidateButton_, grid2);
		
		
		//JPanel texture1ImagePanel = new JPanel();
		GridBagConstraints gbc_texture1ImagePanel = new GridBagConstraints();
		gbc_texture1ImagePanel.insets = new Insets(0, 0, 5, 0);
		gbc_texture1ImagePanel.fill = GridBagConstraints.BOTH;
		gbc_texture1ImagePanel.gridx = 2;
		gbc_texture1ImagePanel.gridy = 7;
		add(texture1ImagePanel_, gbc_texture1ImagePanel,
				DetailPanelMode.BASICS);
		texture1ImagePanel_.setLayout(new GridLayout(0, 1, 0, 0));
		texture1ImagePanel_.add(texture1ImageLabel_);
		
		
		//Texture 2
		JPanel texture2LabelPanel = new JPanel();
		GridBagConstraints gbc_texture2LabelPanel = new GridBagConstraints();
		gbc_texture2LabelPanel.anchor = GridBagConstraints.EAST;
		gbc_texture2LabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_texture2LabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_texture2LabelPanel.gridx = 0;
		gbc_texture2LabelPanel.gridy = 9;
		add(texture2LabelPanel, gbc_texture2LabelPanel,
				DetailPanelMode.BASICS);
		texture2LabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		texture2LabelPanel.add(texture2Label_);
		
		JPanel texture2SelectButtonPanel = new JPanel();
		GridBagConstraints gbc_texture2SelectButtonPanel = new GridBagConstraints();
		gbc_texture2SelectButtonPanel.insets = new Insets(0, 0, 5, 0);
		gbc_texture2SelectButtonPanel.fill = GridBagConstraints.BOTH;
		gbc_texture2SelectButtonPanel.gridx = 1;
		gbc_texture2SelectButtonPanel.gridy = 9;
		add(texture2SelectButtonPanel, gbc_texture2SelectButtonPanel,
				DetailPanelMode.BASICS);
		texture2SelectButtonPanel.setLayout(new GridLayout(2, 1, 0, 0));

		texture2SelectButtonPanel.add(texture2SelectButton_, grid1);
		texture2SelectButtonPanel.add(texture2ValidateButton_, grid2);
		
		
		GridBagConstraints gbc_texture2ImagePanel = new GridBagConstraints();
		gbc_texture2ImagePanel.insets = new Insets(0, 0, 5, 0);
		gbc_texture2ImagePanel.fill = GridBagConstraints.BOTH;
		gbc_texture2ImagePanel.gridx = 2;
		gbc_texture2ImagePanel.gridy = 9;
		add(texture2ImagePanel_, gbc_texture2ImagePanel,
				DetailPanelMode.BASICS);
		texture2ImagePanel_.setLayout(new GridLayout(0, 1, 0, 0));
		texture2ImagePanel_.add(texture2ImageLabel_);
		
		
		setVisibility(activeMode_);

		numScalesSpinner_.addChangeListener(this);
		numHarmonicsSpinner_.addChangeListener(this);
		texture1SelectButton_.addActionListener(this);
		texture2SelectButton_.addActionListener(this);
		texture1ValidateButton_.addActionListener(this);
		texture2ValidateButton_.addActionListener(this);
		
		sequenceFocused(Icy.getMainInterface().getActiveSequence());	
		
	}

	// ----------------------------------------------------------------------------

	/**
	 * Updates the interface to reflect the information of the image. If the
	 * image was already focused, the associated parameters are loaded.
	 */
	public void sequenceFocused(Sequence sequence) {
		activeSequence_ = sequence;
		numScalesSpinner_.removeChangeListener(this);
		numHarmonicsSpinner_.removeChangeListener(this);

		if (sequence != null) {
			selectedImageName_.setText(sequence.getName());
			if (imageLUTs_.containsKey(sequence)) {
				ImageLUTContainer imageLUT = imageLUTs_.get(sequence);
				numScalesSpinner_.setValue(imageLUT.getScale());
				numHarmonicsSpinner_.setValue(imageLUT.getHarmonics().length);
				setTextureWidth(imageLUT.getTextureWidth());
				setTextureHeight(imageLUT.getTextureHeight());
			} else {
				ImageLUTContainer imageLUT = new ImageLUTContainer();
				imageLUT.setSequence(activeSequence_);
				imageLUT.setScales((Integer) numScalesSpinner_.getValue());
				imageLUT.setHarmonics((Integer) numHarmonicsSpinner_.getValue());
				//imageLUT.setTexture1(texture1_, ROITexture1_);
				//imageLUT.setTexture2(texture2_, ROITexture2_);
				imageLUT.setTextureDimension(textureWidth_, textureHeight_);
				imageLUTs_.put(sequence, imageLUT);
			}
		}

		numScalesSpinner_.addChangeListener(this);
		numHarmonicsSpinner_.addChangeListener(this);
	}

	// ----------------------------------------------------------------------------

	/** Erases from internal tables the entries of a particular image. */
	public void sequenceClosed(Sequence sequence) {
		if (sequence != null) {
			imageLUTs_.remove(sequence);
		}
	}

	// ----------------------------------------------------------------------------

	/** Handles the events of the spinner. */
	@Override
	public void stateChanged(ChangeEvent e) {
		if (e.getSource() == numScalesSpinner_
				|| e.getSource() == numHarmonicsSpinner_) {
			if (imageLUTs_.containsKey(activeSequence_)) {
				ImageLUTContainer imageLUT = imageLUTs_.get(activeSequence_);
				imageLUT.setScales((Integer) numScalesSpinner_.getValue());
				imageLUT.setHarmonics((Integer) numHarmonicsSpinner_.getValue());
			} else {
				ImageLUTContainer imageLUT = new ImageLUTContainer();
				imageLUT.setSequence(activeSequence_);
				imageLUT.setScales((Integer) numScalesSpinner_.getValue());
				imageLUT.setHarmonics((Integer) numHarmonicsSpinner_.getValue());
				imageLUT.setTexture1(texture1_, ROITexture1_);
				imageLUT.setTexture2(texture2_, ROITexture2_);
				imageLUTs_.put(activeSequence_, imageLUT);
			}
		}
	}

	// ----------------------------------------------------------------------------

	/** Handles the events of the combo box and button. */
	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == texture1SelectButton_) {
			
			texture1SelectButton_.removeActionListener(this);
			
			IcyBufferedImage image=Icy.getMainInterface().getActiveImage();
			
			//Draw a rectangle in the middle on the image
			drawRect(image, Color.RED);
			
			//Move the rectangle
			IcyCanvas canva=Icy.getMainInterface().getActiveViewer().getCanvas();
			((Canvas2D) canva).getViewComponent().addMouseListener(this);
			((Canvas2D) canva).getViewComponent().addMouseMotionListener(this);
			
		}
		
		if(e.getSource()==texture1ValidateButton_){
			
			IcyCanvas canva=Icy.getMainInterface().getActiveViewer().getCanvas();
			((Canvas2D) canva).getViewComponent().removeMouseListener(this);
			((Canvas2D) canva).getViewComponent().removeMouseMotionListener(this);
			
			int X=(int)(rect_.getPosition().getX());
			int Y=(int)(rect_.getPosition().getY());
			IcyBufferedImage image=Icy.getMainInterface().getActiveImage();
			drawIcon1(image, X,Y);
		  
			if (imageLUTs_.containsKey(activeSequence_)) {
				ImageLUTContainer imageLUT = imageLUTs_.get(activeSequence_);
				imageLUT.setTexture1(texture1_, ROITexture1_);
				imageLUT.setTextureDimension(textureWidth_, textureHeight_);
			} else {
				// adds the sequence and the parameters to the table
				ImageLUTContainer imageLUT = new ImageLUTContainer();
				imageLUT.setSequence(activeSequence_);
				imageLUT.setTexture1(texture1_, ROITexture1_);
				imageLUT.setTexture2(texture2_, ROITexture2_);
				imageLUT.setScales((Integer) numScalesSpinner_.getValue());
				imageLUT.setHarmonics((Integer) numHarmonicsSpinner_.getValue());
				imageLUT.setTextureDimension(textureWidth_, textureHeight_);
				imageLUTs_.put(activeSequence_, imageLUT);
			}
			
			Icy.getMainInterface().getActiveSequence().removeROI(rect_);
			texture1SelectButton_.addActionListener(this);
			
			oneTextureAsBeenValidated_=true;
			
		}
		
		if (e.getSource() == texture2SelectButton_) {

			texture2SelectButton_.removeActionListener(this);
			
			IcyBufferedImage image=Icy.getMainInterface().getActiveImage();
			
			//Draw a rectangle in the middle on the image
			drawRect(image, Color.GREEN);
			
			//Move the rectangle
			IcyCanvas canva=Icy.getMainInterface().getActiveViewer().getCanvas();
			((Canvas2D) canva).getViewComponent().addMouseListener(this);
			((Canvas2D) canva).getViewComponent().addMouseMotionListener(this);
		}
		
		if(e.getSource()==texture2ValidateButton_){
			
			IcyCanvas canva=Icy.getMainInterface().getActiveViewer().getCanvas();
			((Canvas2D) canva).getViewComponent().removeMouseListener(this);
			((Canvas2D) canva).getViewComponent().removeMouseMotionListener(this);
			
			int X=(int)(rect_.getPosition().getX());
			int Y=(int)(rect_.getPosition().getY());
			IcyBufferedImage image=Icy.getMainInterface().getActiveImage();
			drawIcon2(image, X,Y);
		  
			if (imageLUTs_.containsKey(activeSequence_)) {
				ImageLUTContainer imageLUT = imageLUTs_.get(activeSequence_);
				imageLUT.setTexture2(texture2_, ROITexture2_);
				imageLUT.setTextureDimension(textureWidth_, textureHeight_);
			} else {
				// adds the sequence and the parameters to the table
				ImageLUTContainer imageLUT = new ImageLUTContainer();
				imageLUT.setSequence(activeSequence_);
				imageLUT.setTexture1(texture1_, ROITexture1_);
				imageLUT.setTexture2(texture2_, ROITexture1_);
				imageLUT.setScales((Integer) numScalesSpinner_.getValue());
				imageLUT.setHarmonics((Integer) numHarmonicsSpinner_.getValue());
				imageLUT.setTextureDimension(textureWidth_, textureHeight_);
				imageLUTs_.put(activeSequence_, imageLUT);
			}
			
			Icy.getMainInterface().getActiveSequence().removeROI(rect_);
			texture2SelectButton_.addActionListener(this);
			
			oneTextureAsBeenValidated_=true;
		}
	}
	
	// ----------------------------------------------------------------------------
	public void drawRect(IcyBufferedImage image, Color color){
		
//		textureWidth_=95;
//		textureHeight_=95;
		
		int X =(int)(image.getWidth()/2.0)-(int)(textureWidth_/2.0)-1;
		int Y =(int)(image.getHeight()/2.0)-(int)(textureHeight_/2.0)-5;
		
		rect_=new ROI2DRectangle(X,Y,textureWidth_+X,textureHeight_+Y);
		rect_.setColor(color);

		
		Icy.getMainInterface().getActiveSequence().addROI(rect_);
	}
	
	// ----------------------------------------------------------------------------
	public void drawRect2(IcyBufferedImage image, Color color){
		
		int X =250;
		int Y =380;
		
		rect_=new ROI2DRectangle(X,Y,textureWidth_+X,textureHeight_+Y);
		rect_.setColor(color);

		
		Icy.getMainInterface().getActiveSequence().addROI(rect_);
	}
	
	// ----------------------------------------------------------------------------
	public void drawIcon1(IcyBufferedImage image, int X, int Y){
		int width=(int)rect_.getBounds().getWidth();
		int height=(int)rect_.getBounds().getHeight();
		
		//A enlever
//		width=95;
//		height=width;
//
//		X=(int)(image.getWidth()/2.0)-(int)(width/2.0)-1;
//		Y=(int)(image.getHeight()/2.0)-(int)(height/2.0)-5;
//		
//		Rectangle bounds=new Rectangle(width,height);
//		bounds.setLocation(X,Y);
//		rect_.setBounds2D(bounds);
		//
		
		//IcyBufferedImage cropImage = image.getSubImageCopy(X, Y, width,height);
		IcyBufferedImage cropImage =IcyBufferedImageUtil.getSubImage(image, X, Y, width,height);
		texture1_=new IcyBufferedImage(width, height, 1, DataType.DOUBLE) ;
		texture1_=cropImage;
		
		ROITexture1_= rect_;
		
		textureWidth_=width;
		textureHeight_=height;
		
		refreshTexture1ImagePanel();
		
		//Display
//		Sequence sequence = new Sequence();
//		sequence.addImage(texture1_);
//		Icy.getMainInterface().addSequence(sequence);
	}
	
	// ----------------------------------------------------------------------------
	public void drawIcon2(IcyBufferedImage image, int X, int Y){
		int width=(int)rect_.getBounds().getWidth();
		int height=(int)rect_.getBounds().getHeight();
		
		//A enlever
//		X =250;
//		Y =380;
//
//		Rectangle bounds=new Rectangle(width,height);
//		bounds.setLocation(X,Y);
//		rect_.setBounds2D(bounds);
		//
		
		
		//IcyBufferedImage cropImage = image.getSubImageCopy(X, Y, width,height);
		IcyBufferedImage cropImage =IcyBufferedImageUtil.getSubImage(image, X, Y, width,height);
		texture2_=new IcyBufferedImage(width, height, 1, DataType.DOUBLE) ;
		texture2_=cropImage;
		
		ROITexture2_= rect_;
		
		textureWidth_=width;
		textureHeight_=height;
		
		refreshTexture2ImagePanel();
		
		
		//Display
//		Sequence sequence = new Sequence();
//		sequence.addImage(texture2_);
//		Icy.getMainInterface().addSequence(sequence);
	}
		
	// ----------------------------------------------------------------------------
		public void setTextureWidth(int width) {
			textureWidth_=width;
		}
		
	// ----------------------------------------------------------------------------
		public void setTextureHeight(int height) {
			textureHeight_=height;
		}
	
	
	// ============================================================================
	// PRIVATE METHODS
	
	private void refreshTexture2ImagePanel() {
	    texture2ImageLabel_=new JLabel(new ImageIcon(texture2_.getScaledInstance(iconSizeWidth_, iconSizeHeight_, 0)));
	    texture2ImagePanel_.removeAll();
	    texture2ImagePanel_.add(texture2ImageLabel_);
	    this.revalidate();
	}
	
	private void refreshTexture1ImagePanel() {
	    texture1ImageLabel_=new JLabel(new ImageIcon(texture1_.getScaledInstance(iconSizeWidth_, iconSizeHeight_, 0)));
	    texture1ImagePanel_.removeAll();
	    texture1ImagePanel_.add(texture1ImageLabel_);
	    this.revalidate();
	}
	
	// ----------------------------------------------------------------------------

	@Override
	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
	}

	@Override
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
	}

	@Override
	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
	}

	@Override
	public void mouseDragged(MouseEvent arg0) {

		if(!oneTextureAsBeenValidated_){
			Rectangle bounds=new Rectangle((int)rect_.getBounds().getWidth(),(int)rect_.getBounds().getWidth());
			bounds.setLocation((int)rect_.getPosition().getX(), (int)rect_.getPosition().getY());
			rect_.setBounds2D(bounds);//To be squared
		}
		else{
			Rectangle bounds=new Rectangle(textureWidth_,textureHeight_);//In order for the two texture to have the same size
			bounds.setLocation((int)rect_.getPosition().getX(), (int)rect_.getPosition().getY());
			rect_.setBounds2D(bounds);//To be squared
		}
	}

	@Override
	public void mouseMoved(MouseEvent arg0) {
		// TODO Auto-generated method stub
	}

}
