/*******************************************************************************
 * Copyright (c) 2012-2013 Biomedical Image Group (BIG), EPFL, Switzerland.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Ricard Delgado-Gonzalo (ricard.delgado@gmail.com)
 *     Nicolas Chenouard (nicolas.chenouard@gmail.com)
 *     Philippe Th&#233;venaz (philippe.thevenaz@epfl.ch)
 *     Emrah Bostan (emrah.bostan@gmail.com)
 *     Ulugbek S. Kamilov (kamilov@gmail.com)
 *     Ramtin Madani (ramtin_madani@yahoo.com)
 *     Masih Nilchian (masih_n85@yahoo.com)
 *     C&#233;dric Vonesch (cedric.vonesch@epfl.ch)
 *     Virginie Uhlmann (virginie.uhlmann@epfl.ch)
 *     Cl&#233;ment Marti (clement.marti@epfl.ch)
 *     Julien Jacquemot (julien.jacquemot@epfl.ch)
 *     Daniel Schmitter (daniel.schmitter@epfl.ch)
 ******************************************************************************/
package plugins.big.texturesnake.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import plugins.big.bigsnakeutils.icy.gui.CollapsiblePanel;
import plugins.big.bigsnakeutils.icy.gui.DetailPanelMode;
import plugins.big.texturesnake.TextureSnake;
import plugins.big.texturesnake.core.Settings;
import plugins.big.texturesnake.snake.ESnakeEnergyType;
import plugins.big.texturesnake.snake.ESnakeParameters;

/**
 * Panel in which the user specifies the parameters of the E-Snake.
 * 
 * @version May 24, 2015
 * 
 * @author Ricard Delgado-Gonzalo (ricard.delgado@gmail.com)
 * @author Daniel Schmitter (daniel.schmitter@epfl.ch)
 */
public class SnakeSettingsPane extends CollapsiblePanel implements
ChangeListener, ItemListener {

	/**
	 * Determines if a de-serialized file is compatible with this class.
	 * 
	 * Maintainers must change this value if and only if the new version of this
	 * class is not compatible with old versions. See Sun docs for <a
	 * href=http://java.sun.com/products/jdk/1.1/docs/guide
	 * /serialization/spec/version.doc.html> details. </a>
	 * 
	 * Not necessary to include in first version of the class, but included here
	 * as a reminder of its importance.
	 */
	private static final long serialVersionUID = -2078162621447160540L;

	private final JSpinner numControlPointsSpinner_ = new JSpinner();
	private final JComboBox energyTypeComboBox_ = new JComboBox();
	private final JSpinner maxIterationsSpinner_ = new JSpinner();
	private final JCheckBox immortalCheckbox_ = new JCheckBox("Immortal");

	private final JLabel numControlPointsLabel_ = new JLabel("Control points");
	private final JLabel energyTypeLabel_ = new JLabel("Energy type");
	private final JLabel maxIterationsLabel_ = new JLabel("Max iterations");
	private final String[] energyTypeStrings_ = new String[] { "Texture_correlation", "Texture_Fischer" };


	private TextureSnake bigSnake_ = null;

	// ============================================================================
	// PUBLIC METHODS

	/** Default constructor. */
	public SnakeSettingsPane(String title, TextureSnake bigSnake) {
		super(title);
		bigSnake_ = bigSnake;

		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[] { 150, 150};
		gridBagLayout.rowHeights = new int[] { 28, 27, 28, 28};
		gridBagLayout.columnWeights = new double[] { 0.0, 1.0, Double.MIN_VALUE };
		gridBagLayout.rowWeights = new double[] { 0.0, 0.0, 0.0,
				0.0, Double.MIN_VALUE };
		setLayout(gridBagLayout);

		//Control points
		JPanel numControlPointsLabelPanel = new JPanel();
		GridBagConstraints gbc_numControlPointsLabelPanel = new GridBagConstraints();
		gbc_numControlPointsLabelPanel.anchor = GridBagConstraints.EAST;
		gbc_numControlPointsLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_numControlPointsLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_numControlPointsLabelPanel.gridx = 0;
		gbc_numControlPointsLabelPanel.gridy = 0;
		add(numControlPointsLabelPanel, gbc_numControlPointsLabelPanel,
				DetailPanelMode.BASICS);
		numControlPointsLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		numControlPointsLabelPanel.add(numControlPointsLabel_);

		JPanel numControlPointsSpinnerPanel = new JPanel();
		GridBagConstraints gbc_numControlPointsSpinnerPanel = new GridBagConstraints();
		gbc_numControlPointsSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbc_numControlPointsSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbc_numControlPointsSpinnerPanel.gridx = 1;
		gbc_numControlPointsSpinnerPanel.gridy = 0;
		add(numControlPointsSpinnerPanel, gbc_numControlPointsSpinnerPanel,
				DetailPanelMode.BASICS);
		numControlPointsSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));

		numControlPointsSpinnerPanel.add(numControlPointsSpinner_);
		numControlPointsSpinner_.setModel(new SpinnerNumberModel(new Integer(
				Settings.M_DEFAULT), new Integer(3), new Integer(50),
				new Integer(1)));
		
		//Energy type	
		JPanel energyTypeLabelPanel = new JPanel();
		GridBagConstraints gbc_energyTypeLabelPanel = new GridBagConstraints();
		gbc_energyTypeLabelPanel.anchor = GridBagConstraints.EAST;
		gbc_energyTypeLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_energyTypeLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_energyTypeLabelPanel.gridx = 0;
		gbc_energyTypeLabelPanel.gridy = 1;
		add(energyTypeLabelPanel, gbc_energyTypeLabelPanel,
				DetailPanelMode.BASICS);
		energyTypeLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		//energyTypeLabelPanel.add(energyTypeLabel_);

		JPanel energyTypeComboBoxPanel = new JPanel();
		GridBagConstraints gbc_energyTypeComboBoxPanel = new GridBagConstraints();
		gbc_energyTypeComboBoxPanel.insets = new Insets(0, 0, 5, 0);
		gbc_energyTypeComboBoxPanel.fill = GridBagConstraints.BOTH;
		gbc_energyTypeComboBoxPanel.gridx = 1;
		gbc_energyTypeComboBoxPanel.gridy = 1;
		add(energyTypeComboBoxPanel, gbc_energyTypeComboBoxPanel,
				DetailPanelMode.BASICS);
		energyTypeComboBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));

		//energyTypeComboBoxPanel.add(energyTypeComboBox_);
		energyTypeComboBox_.setModel(new DefaultComboBoxModel(
				energyTypeStrings_));
		energyTypeComboBox_.setSelectedIndex(1);


		//Max iteration
		JPanel maxIterationsLabelPanel = new JPanel();
		GridBagConstraints gbc_maxIterationsLabelPanel = new GridBagConstraints();
		gbc_maxIterationsLabelPanel.anchor = GridBagConstraints.EAST;
		gbc_maxIterationsLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_maxIterationsLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_maxIterationsLabelPanel.gridx = 0;
		gbc_maxIterationsLabelPanel.gridy = 1;
		add(maxIterationsLabelPanel, gbc_maxIterationsLabelPanel,
				DetailPanelMode.BASICS);
		maxIterationsLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		maxIterationsLabelPanel.add(maxIterationsLabel_);

		JPanel maxIterationsSpinnerPanel = new JPanel();
		GridBagConstraints gbc_maxIterationsSpinnerPanel = new GridBagConstraints();
		gbc_maxIterationsSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbc_maxIterationsSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbc_maxIterationsSpinnerPanel.gridx = 1;
		gbc_maxIterationsSpinnerPanel.gridy = 1;
		add(maxIterationsSpinnerPanel, gbc_maxIterationsSpinnerPanel,
				DetailPanelMode.BASICS);
		maxIterationsSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));

		maxIterationsSpinnerPanel.add(maxIterationsSpinner_);
		maxIterationsSpinner_
		.setModel(new SpinnerNumberModel(Settings.MAX_LIFE_DEFAULT,
				new Integer(1), null, new Integer(1)));

		//Immortal
		JPanel immortalCheckboxPanel = new JPanel();
		immortalCheckboxPanel.setBorder(null);
		GridBagConstraints gbc_immortalCheckboxPanel = new GridBagConstraints();
		gbc_immortalCheckboxPanel.fill = GridBagConstraints.BOTH;
		gbc_immortalCheckboxPanel.gridx = 1;
		gbc_immortalCheckboxPanel.gridy = 2;
		add(immortalCheckboxPanel, gbc_immortalCheckboxPanel,
				DetailPanelMode.BASICS);
		immortalCheckboxPanel.setLayout(new GridLayout(0, 1, 0, 0));

		immortalCheckbox_.setSelected(false);
		maxIterationsSpinner_.setEnabled(!immortalCheckbox_.isSelected());

		immortalCheckboxPanel.add(immortalCheckbox_);

		setVisibility(activeMode_);

		numControlPointsSpinner_.addChangeListener(this);
		energyTypeComboBox_.addItemListener(this);
		maxIterationsSpinner_.addChangeListener(this);
		immortalCheckbox_.addItemListener(this);
	}

	// ----------------------------------------------------------------------------

	/** Returns the energy type selected by the user. */
	public ESnakeEnergyType getEnergyType() {
		switch (energyTypeComboBox_.getSelectedIndex()) {
		case 0:
			return ESnakeEnergyType.TEXTURE_COR;
		case 1:
			return ESnakeEnergyType.TEXTURE_FISH;
		}
		return null;
	}

	// ----------------------------------------------------------------------------

	/** Returns the maximum number of iterations introduced by the user. */
	public Integer getMaxIterations() {
		return (Integer) maxIterationsSpinner_.getValue();
	}

	// ----------------------------------------------------------------------------

	/** Returns the number of control points introduced by the user. */
	public Integer getNumControlPoints() {
		return (Integer) numControlPointsSpinner_.getValue();
	}

	// ----------------------------------------------------------------------------

	/**
	 * Returns <code>true</code> if the user wants to ignore the maximum number
	 * of iterations, and evolve the snake till convergence. Returns
	 * <code>false</code> otherwise.
	 */
	public boolean isImmortal() {
		return immortalCheckbox_.isSelected();
	}

	// ----------------------------------------------------------------------------

	/** Sets the snake parameters to the interface. */
	public void setSnakeParameters(ESnakeParameters snakeParameters) {
		numControlPointsSpinner_.setValue(snakeParameters.getM());
		maxIterationsSpinner_.setValue(snakeParameters.getMaxLife());
		immortalCheckbox_.setSelected(snakeParameters.isImmortal());
		switch (snakeParameters.getEnergyType()) {
		case TEXTURE_COR:
			energyTypeComboBox_.setSelectedIndex(0);
			break;
		case TEXTURE_FISH:
			energyTypeComboBox_.setSelectedIndex(1);
			break;
		}
	}

	// ============================================================================
	// LISTENER METHODS

	/** Handles the events of the spinners. */
	@Override
	public void stateChanged(ChangeEvent e) {
		Object source = e.getSource();
		if (source == numControlPointsSpinner_ || source == maxIterationsSpinner_) {
			bigSnake_.loadSnakeParametersFromInterface();
		}
	}

	// ----------------------------------------------------------------------------

	/**
	 * Handles the events of the combo boxes and the checkbox. It fires an
	 * update of the active snake.
	 */
	@Override
	public void itemStateChanged(ItemEvent e) {
		if (e.getSource() == energyTypeComboBox_) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				bigSnake_.loadSnakeParametersFromInterface();
			}
		} else if (e.getSource() == immortalCheckbox_) {
			maxIterationsSpinner_.setEnabled(!immortalCheckbox_.isSelected());
			bigSnake_.loadSnakeParametersFromInterface();
		}
	}
}
