package waveletpackage;

import fftacademic.AcademicFFT;
import image2d.Image2DDouble;
//import imageware.Builder;

/**
 * This class performs the circular harmonic wavelets decomposition of an image.
 */
public class AnalysisR {
	
	private WaveletFiltersSimoncelli		filters_;
	private Image2DDouble[] 				multiplierCH_real;
	private Image2DDouble[] 				multiplierCH_imag;

	
	private int					nx;
	private int					ny;
	private int					nscales_;
	private int 			    nchannels_;
	
	double[] 					image = null;
	
	
	double[] 					res_re = null;
	double[] 					res_im = null;
	double[][] 					wd_re = null;
	double[][] 					wd_im = null;
	double[]				  	wa_re = null;
    double[] 					wa_im = null;
    double[][][] 				wd_chan_re = null;   
    double[][][] 				wd_chan_im = null;
    
	
	public AnalysisR(double[] image, int nx, int ny, int[] harmonics, int nscales) {
		
		this.nchannels_ = harmonics.length;

		this.image = image;
		this.nx = nx;
		this.ny = ny;
		this.nscales_ = nscales;
	
		filters_ = new WaveletFiltersSimoncelli(nx, ny);

		MultipliersR.channelMult(harmonics, nx, ny);
		multiplierCH_real = MultipliersR.getRealFilter();
		multiplierCH_imag = MultipliersR.getImagFilter();		
	}
	
	public double[] getRes_re(){
		return res_re;
	}
	public double[] getRes_im(){
		return res_im;
	}
	public double[][][] getWdChan_re(){
		return wd_chan_re;
	}
	public double[][][] getWdChan_im(){
		return wd_chan_im;
	}
	public double[] getWa_re(){
		return wa_re;
	}
	public double[] getWa_im(){
		return wa_im;
	}

	public void analyze() {

		double[] imData = new double[image.length];
		AcademicFFT fft = new AcademicFFT(nx, ny, 0, 0);
		fft.directTransform(image, imData, null, null, AcademicFFT.InputDataType.REALINPUT);
		

		double R[] = MathSymmetry.buildRF(nx, ny);

		// construct prefilters

		double[] preLP = filters_.simoncelliD(R, false, 1);
		double[] preHP = filters_.simoncelliD(R, true, 1);
		

		// compute residual and filtered image

		res_re = multiply(image, preHP);
		res_im = multiply(imData, preHP);

		wa_re = multiply(image, preLP);
		wa_im = multiply(imData, preLP);


		double[] LP, HP;
		wd_re = new double[nscales_][image.length];
		wd_im = new double[nscales_][imData.length];
		
		for (int sc = 0; sc < nscales_; ++sc) {
			LP  = filters_.simoncelliD(R, false, Math.pow(2, -(sc+1)));
			HP = filters_.simoncelliD(R, true, Math.pow(2,-(sc+1)));

			// construct wavelet band and lowpass image

			wd_re[sc] = multiply(HP,wa_re);
			wd_im[sc] = multiply(HP,wa_im); 
			
			wa_re = multiply(LP,wa_re);
			wa_im = multiply(LP,wa_im);
			
		}

		wd_chan_re = new double[nscales_][nchannels_][image.length];
		wd_chan_im = new double[nscales_][nchannels_][imData.length];
		
		for (int sc = 0; sc < nscales_; ++sc) {
			for (int ii = 0; ii < nchannels_; ++ii) {

				wd_chan_re[sc][ii] = computeRealChan(wd_re[sc], wd_im[sc], multiplierCH_real[ii], multiplierCH_imag[ii]);
				wd_chan_im[sc][ii] = computeImagChan(wd_re[sc], wd_im[sc], multiplierCH_real[ii], multiplierCH_imag[ii]);
				
				//Builder.create(transform(nx, ny, wd_chan_re[sc][ii])).show("real ch "+ii);
				
			}
		}

	}

	private double[] computeImagChan(double[] wd_real, double[] wd_imag, Image2DDouble mult_re, Image2DDouble mult_im){

		double[] mult_real = mult_re.getData();
		double[] mult_imag = mult_im.getData();
		
		double[] c = new double[wd_real.length];
		for(int i=0; i<wd_real.length; ++i){
			c[i] =  wd_real[i]*mult_imag[i] + wd_imag[i]*mult_real[i];
		}
		return c;

	}
	
	private double[] computeRealChan(double[] wd_real, double[] wd_imag, Image2DDouble mult_re, Image2DDouble mult_im){

		double[] mult_real = mult_re.getData();
		double[] mult_imag = mult_im.getData();
		
		double[] c = new double[wd_real.length];
		
		for(int i=0; i<wd_real.length; ++i){
			c[i] =  wd_real[i]*mult_real[i] - wd_imag[i]*mult_imag[i];
		}
		return c;

	}

	private double[] multiply(double[] data, double[] filter) {
		double[] a = new double[data.length];
		for(int i=0; i<data.length; ++i){
			a[i] = data[i]*filter[i];
		}
		return a;
	}

	private double[][] transform(int nx, int ny, double[] data){
		int offset = 0;
		int leni = nx;
		int lenj = ny;
		double[][] buffer = new double[nx][ny];
		for (int j = 0; j < lenj; j++) {
			offset = j * nx;
			for (int i = 0; i < leni; i++, offset++) {
				buffer[i][j] = data[offset];
			}
		}

		return buffer;
	}

}
