package waveletpackage;


public class MathSymmetry {

	/**
	 * Create a linear vector
	 * @param min
	 * @param max
	 * @param num
	 * @return
	 */
	public static double[] linspace(double min, double max, int num) {
		double[] vec = new double[num];
		vec[0] = min;
		double increment = (max - min) / (double) num;
		for (int i = 1; i < num; ++i){
			vec[i] = vec[i-1]+increment;
			}
		return vec;
	}


	/**
	 * Return the angle values for different x and y values
	 * @param nx
	 * @param ny
	 * @return
	 */
	public static double[][] buildAngleMap(int nx, int ny){
	
		double[] x = linspace(-Math.PI, Math.PI, nx);
		double[] y = linspace(-Math.PI, Math.PI, ny);
		double[][] dist = new double[nx][ny];
		for(int i=0; i<nx; ++i){
			for(int j=0; j<ny; ++j){
				dist[i][j] = Math.atan2(x[i],y[j]);			
			}
		}
		return dist;
	}

	public static double[] buildRF(int nx, int ny) {
		double[] x = linspace(0, 1, nx);
		double[] y = linspace(0, 1, ny);
		double[] dist = new double[nx*ny];
		for (int i = 0; i < nx; ++i)
		for (int j = 0; j < ny; ++j) {
			double val1 = x[i] - Math.floor(2.0*x[i]);
			double val2 = y[j] - Math.floor(2.0*y[j]);
			dist[i + nx*j] = Math.sqrt(val1*val1 + val2*val2);
		}
		return dist;
	}
	
	public static double[][] buildRF2D(int nx, int ny) {
		double[] x = linspace(0, 1, nx);
		double[] y = linspace(0, 1, ny);
		double[][] dist = new double[nx][ny];
		for (int i = 0; i < nx; ++i)
		for (int j = 0; j < ny; ++j) {
			double val1 = x[i] - Math.floor(2.0*x[i]);
			double val2 = y[j] - Math.floor(2.0*y[j]);
			dist[i][j] = Math.sqrt(val1*val1 + val2*val2);
		}
		return dist;
	}

	public static double log2(double val) {
		return Math.log(val) / Math.log(2);
	}

}
