package waveletpackage;

import fftacademic.AcademicFFT;
import image2d.Image2DDouble;

public class SynthesisR {
	
	private WaveletFiltersMeyer				filters_;
	private Image2DDouble[] 				multiplierCH_real;
	private Image2DDouble[] 				multiplierCH_imag;
	
	private int					nx;
	private int					ny;
	private int					nscales_;
	private int 			    nchannels_;
	
	double[] res_re = null;
	double[] res_im = null;
	double[] wa_re = null;
    double[] wa_im = null;
    double[][][] wd_chan_re = null;   
    double[][][] wd_chan_im = null;
    
    
	public SynthesisR(int nx, int ny, double[][][] wd_chan_re, double[][][] wd_chan_im, double[] wa_re, double[] wa_im, double[] res_re, double[] res_im, int[] harmonics){
		
		this.nx = nx;
		this.ny = ny;
		this.nscales_ = wd_chan_re.length;
		this.nchannels_ = wd_chan_re[0].length;
		
		this.wd_chan_re = wd_chan_re;
		this.wd_chan_im = wd_chan_im;
		this.wa_re = wa_re;
		this.wa_im = wa_im; 
		this.res_re = res_re;
		this.res_im = res_im;
		
		filters_ = new WaveletFiltersMeyer(nx, ny);
		
		MultipliersR.channelMult(harmonics, nx, ny);
		multiplierCH_real = MultipliersR.getRealFilter();
		multiplierCH_imag = MultipliersR.getImagFilter();
		
	}
	
	public double[] getImg_re(){
		return wa_re;
	}
	public double[] getImg_im(){
		return wa_im;
	}
	
	public void Synthesize(){
		
		AcademicFFT fft = new AcademicFFT(nx, ny, 0, 0);
		
		// combine channels
		
		double[][] wd_re = new double[nscales_][nx*ny];
		double[][] wd_im = new double[nscales_][nx*ny];

		for(int sc=0; sc<nscales_; ++sc){
			for(int ii=0; ii<nchannels_; ++ii){
				wd_re[sc] = computeRealChan(wd_chan_re[sc][ii],wd_chan_im[sc][ii],multiplierCH_real[ii],multiplierCH_imag[ii]);
				wd_im[sc] = computeImagChan(wd_chan_re[sc][ii],wd_chan_im[sc][ii],multiplierCH_real[ii],multiplierCH_imag[ii]);
			}			
		}
	
		
		// combine scale with lowpass
		
		double[] LP, HP;
		double R[] = MathSymmetry.buildRF(nx, ny);
		
		
		for(int sc=nscales_-1; sc>-1; --sc){
			
			// construct filters
			LP = filters_.meyerD(R, false, Math.pow(2, -(sc+1)));
			HP = filters_.meyerD(R, true, Math.pow(2, -(sc+1)));
			
			// update image
		    
			wa_re = add( multiply(wa_re,LP), multiply(wd_re[sc],HP));
			wa_im = add( multiply(wa_im,LP), multiply(wd_im[sc],HP));
			
		}
		
		// construct prefilters
		
		double[] preLP = filters_.meyerD(R, false, 1);
		double[] preHP = filters_.meyerD(R, true, 1);
		
	    // combine with residuals
		
		wa_re = add(multiply(wa_re,preLP), multiply(res_re,preHP));
		wa_im = add(multiply(wa_im,preLP), multiply(res_im,preHP));
	    
	    // IFFT
		
		fft.inverseTransform(wa_re, wa_im, null, null);
		
	}
	
	
	private double[] computeImagChan(double[] wd_chan_re, double[] wd_chan_imag, Image2DDouble mult_re, Image2DDouble mult_im){

		double[] mult_real = mult_re.getData();
		double[] mult_imag = mult_im.getData();
		
		double[] c = new double[wd_chan_re.length];
		for(int i=0; i<wd_chan_re.length; ++i){
			c[i] =  wd_chan_imag[i]*mult_real[i] - wd_chan_re[i]*mult_imag[i];
		}
		return c;

	}
	
	private double[] computeRealChan(double[] wd_chan_re, double[] wd_chan_imag, Image2DDouble mult_re, Image2DDouble mult_im){

		double[] mult_real = mult_re.getData();
		double[] mult_imag = mult_im.getData();
		
		double[] c = new double[wd_chan_imag.length];
		
		for(int i=0; i<wd_chan_imag.length; ++i){
			c[i] =  wd_chan_re[i]*mult_real[i] + wd_chan_imag[i]*mult_imag[i];
		}
		return c;

	}

	private double[] add(double[] a, double[] b) {
	
		double[] c = new double[a.length];
		for(int i=0; i<a.length; ++i){
			c[i] = a[i]+b[i];
		}
		
		return c;
	}
	
	private double[] multiply(double[] data, double[] filter) {
		double[] a = new double[data.length];
		for(int i=0; i<data.length; ++i){
			a[i] = data[i]*filter[i];
		}
		return a;
	}


}
