package waveletpackage;

import image2d.Image2DDouble;

public class WaveletFiltersMeyer {

	private double	LE_		= 0.25; // left endpoint of transition
	private double	RE_		= 0.5;		// right endpoint of transition
	
	private double root1 	= 0.125;
	private double root2	= 0.25;
	
	double c_4 = 1;
	double c_3 = -2*(root1+root2);
	double c_2 = root1*root1+4*root1*root2+root2*root2;
	double c_1 = -2*(root2*root1*root1+root1*root2*root2);
	double c_0 = root1*root1*root2*root2;

	private int		nx_;
	private int		ny_;

	public WaveletFiltersMeyer(int nx, int ny) {
		this.ny_ = ny;
		this.nx_ = nx;
	}
	
	public Image2DDouble meyer(double R[], boolean lh, double cutoff) {
		Image2DDouble y = new Image2DDouble(nx_, ny_);
		for (int i = 0; i < nx_; ++i) {
			for (int j = 0; j < ny_; ++j) {
				y.putPixel(i, j, meyerprofile(R[i*ny_ +j], lh, cutoff));
			}
		}
		return y;
	}

	public Image2DDouble meyer(double R[][], boolean lh, double cutoff) {
		Image2DDouble y = new Image2DDouble(nx_, ny_);
		for (int i = 0; i < nx_; ++i) {
			for (int j = 0; j < ny_; ++j) {
				y.putPixel(i, j, meyerprofile(R[i][j], lh, cutoff));
			}
		}
		return y;
	}

	public double[] meyerD(double R[], boolean lh, double cutoff){
		double[] y = new double[nx_*ny_];
		for (int i = 0; i < nx_; ++i) {
			for (int j = 0; j < ny_; ++j) {
				y[i*ny_ +j] = meyerprofile(R[i*ny_+j], lh, cutoff);
			}
		}
		return y;
	}
	
	public double[] meyerD(double R[][], boolean lh, double cutoff){
		double[] y = new double[nx_*ny_];
		for (int i = 0; i < nx_; ++i) {
			for (int j = 0; j < ny_; ++j) {
				y[i*ny_ +j] = meyerprofile(R[i][j], lh, cutoff);
			}
		}
		return y;
	}

	public double meyerprofile(double xval, boolean lh, double cutoff) {

		// rescale for cutoff value
		double x = xval / cutoff;
		double y = 0;
		// low-pass (lh=0) and high-pass (lh=1) filters
		if (!lh) {
			if (x <= LE_) {
				y = 1.0;
			}
			if ((x > LE_) & (x <= RE_)) {
			    y = Math.sin((Math.PI/2)*q3(x/2.0));

			}
			if (x > RE_) {
				y = 0.0;
			}

		}
		else {
			if (x <= LE_) {
				y = 0.0;
			}
			if ((x > LE_) & (x <= RE_)) {
				y = Math.cos((Math.PI/2)*q3(x/2.0));
			}
			if (x > RE_) {
				y = 1.0;
			}
		}
		return y;

	}

	
	private double q3(double x){
		return q2(x)/q2(root1);
	}

	private double q2(double x){
			return q1(x) - q1(root2);
	}
	
	private double q1(double x){
		return 0.2*c_4*Math.pow(x,5) + 0.25*c_3*Math.pow(x,4) + (1.0/3.0)*c_2*Math.pow(x,3) +0.5*c_1*x*x + c_0*x; 
	}
	
}
