package waveletpackage;

import image2d.Image2DDouble;

public class WaveletFiltersSimoncelli {

	private double	LE_		= 0.25; // left endpoint of transition
	private double	RE_		= 0.5;		// right endpoint of transition

	private int		nx_;
	private int		ny_;

	public WaveletFiltersSimoncelli(int nx, int ny) {
		this.ny_ = ny;
		this.nx_ = nx;
	}
	
	public Image2DDouble simoncelli(double R[], boolean lh, double cutoff) {
		Image2DDouble y = new Image2DDouble(nx_, ny_);
		for (int i = 0; i < nx_; ++i) {
			for (int j = 0; j < ny_; ++j) {
				y.putPixel(i, j, simoncelliprofile(R[i*ny_ +j], lh, cutoff));
			}
		}
		return y;
	}

	public Image2DDouble simoncelli(double R[][], boolean lh, double cutoff) {
		Image2DDouble y = new Image2DDouble(nx_, ny_);
		for (int i = 0; i < nx_; ++i) {
			for (int j = 0; j < ny_; ++j) {
				y.putPixel(i, j, simoncelliprofile(R[i][j], lh, cutoff));
			}
		}
		return y;
	}

	public double[] simoncelliD(double R[], boolean lh, double cutoff){
		double[] y = new double[nx_*ny_];
		for (int i = 0; i < nx_; ++i) {
			for (int j = 0; j < ny_; ++j) {
				y[i*ny_ +j] = simoncelliprofile(R[i*ny_+j], lh, cutoff);
			}
		}
		return y;
	}
	
	public double[] simoncelliD(double R[][], boolean lh, double cutoff){
		double[] y = new double[nx_*ny_];
		for (int i = 0; i < nx_; ++i) {
			for (int j = 0; j < ny_; ++j) {
				y[i*ny_ +j] = simoncelliprofile(R[i][j], lh, cutoff);
			}
		}
		return y;
	}

	public double simoncelliprofile(double xval, boolean lh, double cutoff) {

		// rescale for cutoff value
		double x = xval / cutoff;
		double y = 0;
		// low-pass (lh=0) and high-pass (lh=1) filters
		if (!lh) {
			if (x <= LE_) {
				y = 1.0;
			}
			if ((x > LE_) & (x <= RE_)) {
				y = Math.cos((Math.PI / 2.0) * log2(4*x));
			}
			if (x > RE_) {
				y = 0.0;
			}

		}
		else {
			if (x <= LE_) {
				y = 0.0;
			}
			if ((x > LE_) & (x <= RE_)) {
				y = Math.cos((Math.PI / 2.0) * log2(2*x));
			}
			if (x > RE_) {
				y = 1.0;
			}
		}
		return y;

	}

	private double log2(double x){
		return Math.log(x)/Math.log(2);
	}
	
}
